.renderedMarkdown {
    border: 1px solid #aaa;
    border-radius: 3px;
    padding: 1.5em;
    background: #fff;
    color: #484848;
    margin-bottom: 2em;
    position: relative;
    box-shadow: 0px 2px 3px rgba(0,0,0,0.1);
    font-size: 12pt;
}

@media (max-width: 992px) {
    .renderedMarkdown {
        border-radius: 0px;
        border-left: none;
        border-right: none;
        margin-left: -25px;
        margin-right: -25px;
    }
}

.renderedMarkdown span.title,
.renderedMarkdown h1,
.renderedMarkdown h2,
.renderedMarkdown h3,
.renderedMarkdown h4,
.renderedMarkdown h5,
.renderedMarkdown h6 {
  color: black;
  margin-top: 1.5em;
  margin-bottom: 0.5em;
  font-weight: bold;
}

.renderedMarkdown span.title:nth-of-type(1) {
  margin-top: 1em;
  text-align: center;
  display: block;
  font-size: 2.2em;
  margin-top: .67em;
  margin-bottom: .3em;
}

.renderedMarkdown span.subtitle {
  text-align: center;
  display: block;
  font-size: 1.5em;
  font-style: italic;
  margin-bottom: .3em;
}

.renderedMarkdown h1 {
    border-bottom: 1px solid #eee;
    padding-bottom: 0.3em;
}

.renderedMarkdown div.metadata {
    margin-bottom: 1em;
}

.renderedMarkdown .authors {
    margin: auto;
    display: block;
    width: 80%;
    text-align: center;
    font-size: 1.5em;
    padding-bottom: 0.8em;
}

.renderedMarkdown .date_created {
    display: block;
    margin: auto;
    text-align: center;
    font-size: 1.2em;
}

.renderedMarkdown .date_updated {
    display: block;
    margin: auto;
    text-align: center;
    font-size: 0.8em;
    font-style: italic;
}

.renderedMarkdown .tldr {
    width: 80%;
    padding-top: 1em;
    margin: auto;
    display: block;
    font-style: italic;
    text-align: justify;
}

.renderedMarkdown .tags {
    width: 80%;
    display: block;
    margin: auto;
}

.renderedMarkdown img {
    display: block;
    margin: auto;
    max-width: 80%;
}

.renderedMarkdown .table-wrapper {
    overflow-x: auto;
    display: block;
}

.renderedMarkdown table {
    margin-left: auto;
    margin-right: auto;
    margin-top: 1em;
    margin-bottom: 1em;
    border-collapse: collapse;
    border-spacing: 0;
    color: #000;
    table-layout: fixed;
    border: 1px solid #aaa;
    padding: 2px;
    border-collapse: separate;
}
.renderedMarkdown thead th {
    border-left: 1px solid #ddd;
    border-bottom: 1px solid #ddd;
}
.renderedMarkdown thead th:first-child {
    border-left: none;
}
.renderedMarkdown thead > tr:last-child > th {
    border-bottom: 1px solid #aaa;
    vertical-align: bottom;
}
.renderedMarkdown tr, th, td {
    text-align: center;
    vertical-align: middle;
    padding: 0.5em 0.5em;
    line-height: normal;
    white-space: normal;
    max-width: none;
    border: none;
}
.renderedMarkdown th {
    font-weight: bold;
    text-align: right;
}
.renderedMarkdown thead th {
    text-align: center;
}
.renderedMarkdown tr th {
    background: rgba(255,255,255,0.5);
}
.renderedMarkdown tr > td:first-child:empty {
    background: #fff;
}
.renderedMarkdown tr > td:first-child:empty + th {
    border-left: 1px solid #aaa;
}
.renderedMarkdown tr > th + td {
    border-left: 1px solid #aaa;
}
.renderedMarkdown tr > td + th {
    border-left: 1px solid #000;
}
.renderedMarkdown tbody tr:nth-child(odd) {
    background: #f5f5f5;
}
.renderedMarkdown tbody tr:hover {
    background: rgba(66, 165, 245, 0.2);
}
/* Footnotes for tables generated from other sources via pandoc are in a .footnotes div */
.renderedMarkdown .footnotes {
    background: #eee;
    border: 1px solid #aaa;
    border-top: none;
    margin-top: -1em;
    padding: 0.5em;
}
.renderedMarkdown .footnotes hr {
    display: none;
}

.renderedMarkdown .underline {
    text-decoration: underline;
}

.renderedMarkdown .codehilite {
  margin-top: 25px;
  margin-bottom: 25px;
}

.renderedMarkdown .code-output {
  background: #fafafa;
  padding: 15px;
  overflow: auto;
  border-left: solid #6BD2CA 10px;
}

.renderedMarkdown .codehilite + .code-output  {
  margin-top: -1em;
  margin-left: -1.5em;
  margin-right: -1.5em;
  font-family: monospace;
}

.renderedMarkdown .pre {
  white-space: pre;
}


/* Code highlighting */
.codetoggle, .codetoggleall {
  background-color: #ccc;
  color: #FFFFFF;
  width: 6em;
  padding: 3px;
  text-align: center;
  font-family: sans-serif;
  border-top-right-radius: 4px;
  border-bottom-left-radius: 4px;
  position: absolute;
  top: 0px;
  right: 0px;
  cursor: pointer;
  transition: background-color 0.3s;
}

.codetoggleall {
  width: 140px;
  border-top-right-radius: 0px;
  border-radius: 3px;
  top: -35px;
  z-index: 3;
}

.codetoggle:hover, .codetoggleall:hover {
  background-color: #00a699;
  color: #fff;
    cursor: pointer;
  opacity: 1;
  transition: background-color 0.2s;
}

.codetoggle:active, .codetoggleall:active {
  background: #edd114;
}

@media (max-width: 992px) {
    .codetoggle, .codetoggleall {
        display:none;
    }
}

.renderedMarkdown ul > li > p {
  margin-left: 0;
}

.renderedMarkdown .codehilite {
    margin: 1em;
    margin-left: -1.5em;
    margin-right: -1.5em;
    padding: 0px;
}

.renderedMarkdown .codehilite pre {
    border: none;
    border-left: solid #00a699 10px;
    margin: 0px;
    border-radius: 0px;
}

.renderedMarkdown .codehilite .codetoggle {
    display: flex;
    left: 100%;
    border-radius: 0px;
    border-top-right-radius: 5px;
    border-bottom-right-radius: 5px;
    opacity: 0;
    height: 100%;
    transition: opacity 0.2s;
    font-size: 1.5em;
    width: 0.8em;
    justify-content: center;
    flex-direction: column;
}

.renderedMarkdown .codehilite:hover .codetoggle {
    opacity: 1;
}

.renderedMarkdown .footnote {
    margin-top: 1.5em;
    margin-left: -1.5em;
    margin-right: -1.5em;
    margin-bottom: -1.5em;
    background: #eee;
    border-top: 1px solid #ddd;
    padding: 0.5em;
    border-bottom-left-radius: 3px;
    border-bottom-right-radius: 3px;
}
.renderedMarkdown .footnote hr {
    display: none;
}

.side-panel .btn-download {
    background: #ccc;
    color: #fff;
    border: none;
    transition: background-color 0.3s;
    margin: 5px;
    overflow: hidden;
}

.side-panel .btn-download:hover {
    background: #00a699;
    color: #fff;
    opacity: 1;
    transition: background-color 0.2s;
}

.side-panel .toc ul.nav {
    list-style-type: decimal;
}

.side-panel .toc ul.nav > li {
    display: list-item;
    margin-left: 1.5em;
    font-weight: normal;
}

.side-panel .toc ul.nav > li > a {
    padding: 3px;
}

.side-panel .toc ul.nav > li.active {
    font-weight: bold;
}
