/* This file defines the css styles for the index route */

#feed-posts {
    margin-top: 2em;
    margin-bottom: 2em;
}

.feed-post {
    padding: 10px;
    border-radius: 10px;
    cursor: pointer;
    border: 1px solid transparent;
    transition: background 0.6s, border 0.6s;
    display: flex;
    margin-top: 1em;
    margin-bottom: 1em;
}
.feed-post:hover {
    background: #fff;
    border: 1px solid #00a699;
    transition: background 0.4s, border 0.4s;
}
.feed-post:active {
    transform: scale(.99,0.99);
}

.feed-post .feed-thumbnail {
    height: 200px;
    flex-shrink: 0;
    flex-grow: 0;
    width: 200px;
    position: relative;
    vertical-align: center;
    background: #fff;
    border: 1px solid #bbb;
    border-radius: 5px;
    box-shadow: 0px 1px 3px rgba(0,0,0,0.2);
    margin: 0px;
    transition: border 0.4s, box-shadow 0.4s;
}
.feed-post .feed-thumbnail img {
    position: absolute;
    top: 0px;
    right: 0;
    bottom: 0;
    left: 0;
    max-width:150px;
    max-height: 150px;
    height:auto;
    width:auto;
    margin: auto;
}
.feed-post:hover .feed-thumbnail {
    border: transparent;
    box-shadow: none;
    transition: border 0.6s, box-shadow 0.6s;
}

.feed-post .feed-metadata {
    position:relative;
    margin: 0px;
    margin-left: 1em;
    width: 100%;
}
.feed-post .feed-metadata span {
    display:  block;
}

.feed-post .feed-title {
    margin-top: 0px;
    font-size: 1.6em;
    font-weight: bolder;
}

.feed-post .feed-authors, .feed-post .feed-tags {
  margin-bottom: .3em;
}

.feed-post .feed-tldr {
    height: 5em;
    padding: 10px;
    background: rgba(0,0,0,0.03);
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
    font-style: italic;
    text-align: justify;
    overflow: hidden;
}
.feed-post .feed-tldr-expander {
    padding: 3px;
    display: block;
    text-align: center;
    font-size: small;
    background: rgba(0,0,0,0.03);
    border-bottom-left-radius: 5px;
    border-bottom-right-radius: 5px;
    color: #00a699;
    margin-bottom: 0.3em;
    }
.feed-post .feed-tldr-expander:hover {
    background: rgba(0,0,0,0.07);
}

.feed-post .feed-dates {
    font-style: italic;
    color: #aaa;
    font-size: small;
    margin-top: 3px;
}

.feed-post .feed-stats {
    display: inline-block;
    float: right;
    margin-top: 3px;
    text-align:center;
}
.feed-post .feed-stats span {
  display: inline;
  margin-left: 1em;
  color: #aaa;
}

@media (max-width: 650px) {
    .feed-post {
        flex-direction: column;
        border-radius: 0px;
        margin-left: -25px;
        margin-right: -25px;
        margin-bottom: 10px;
        border-bottom: 1px solid #ddd;
    }

    .feed-post .feed-thumbnail {
        align-self: center;
        margin-bottom: 1em;
        width: 150px;
        height: 150px;
    }

    .feed-post .feed-thumbnail img {
        align-self: center;
        margin-bottom: 1em;
        max-width: 125px;
        max-height: 125px;
    }

    .feed-post .feed-metadata {
        margin-left: 0px;
    }
}
